#!/usr/bin/ruby

# This is a utility for mapping non-ASCII characters to their closest
# ASCII equivalent. 

@repls = [ ]
@repls[193] = 'I'
@repls[197] = 'A'
@repls[198] = 'A'
@repls[201] = 'E'
@repls[205] = 'I'
@repls[214] = 'O'
@repls[216] = 'O'
@repls[218] = 'U'
@repls[223] = 'ss'
@repls[224] = 'a'
@repls[225] = 'a'
@repls[226] = 'a'
@repls[227] = 'a'
@repls[228] = 'a'
@repls[229] = 'a'
@repls[230] = 'a'
@repls[231] = 'c'
@repls[232] = 'e'
@repls[233] = 'e'
@repls[234] = 'e'
@repls[235] = 'i'
@repls[236] = 'i'
@repls[237] = 'i'
@repls[238] = 'i'
@repls[239] = 'i'
@repls[241] = 'n'
@repls[242] = 'o'
@repls[243] = 'o'
@repls[244] = 'o'
@repls[246] = 'o'
@repls[248] = 'o'
@repls[250] = 'u'
@repls[251] = 'u'
@repls[252] = 'u'
@repls[253] = 'y'

# If the @repl array above fails to map x to some character, this
# routine finds the closest possibilities
def find_nearest(x)
  y = x
  s = []
  while x >= 0 do
    if @repls[x] then
      s.push(@repls[x])
      break
    else
      x = x-1
    end
  end
  while y < 254 do
    if @repls[y] then
      s.push(@repls[y])
      break
    else
      y = y + 1
    end
  end
  return s
end

# replace all occurrences of non-ASCII characters in string s with
# ASCII equivalents, according to the @repl array
def repl(s) 
  while s =~ /\&\#\d\d\d;/ do
    m = $&
    d = m[2..(m.length-2)].to_i
    c = @repls[d]
    if c == nil then
      possibilities = find_nearest(d)
      suggestion = ""
      possibilities.each do |sg|
        if suggestion == "" then
          suggestion = sg
        else
          suggestion = suggestion + " or " + sg
        end
      end
      puts "no mapping for digit #{d} in #{s}; perhaps #{suggestion}?"
      return nil
    else
      s = s.sub(/(\&\#\d\d\d;)/,c)
    end
  end
  s
end

##################### MAIN PROGRAM #########################

if ARGV.length != 1 then
  puts "usage: getauthors.rb <directory>"
  puts "  <directory> should contain XML files generated by getpubdir.rb"
  exit 1
end

# Go to the directory containing the venue data
venue = ARGV[0]
if !Dir.chdir(venue) then
  puts "directory #{venue} does not exist"
  exit 2
end

# For each XML file in this directory extract the author and
# associate the author with the relevant conference, printed to
# stdout
files = `\ls *.xml`
files.each do |file|
  File.open(file.chomp!,"r") do |h|
    #puts "processing #{file}"
    file =~ /\d\d/
    paper = venue + ":" + $&
    while line = h.gets do
      #puts "processing #{line}"
      if line =~ /<author>.*<\/author>/
        s = $&
        s = repl(s[8..(s.length-10)])
        if !s then
          puts "need to fix getauthors.rb mapping for non-Latin character"
          exit 1
        end
        puts s + ", " + paper.upcase
      end
    end
  end
end

Dir.chdir("..")
