#!/usr/bin/ruby
# -*- coding: utf-8 -*-
# Usage: getpubdir <venue abbreviation>
# Requires that you have wget installed on your system (tested with
#   version 1.12 on MacOS 10.6)

# file download
def wget(args,url)
  cmd = "wget " + args + " " + url
  return system(cmd) 
end

# Get many files; non-verbose
def wget_files(urls)
  urls.each do |file|
    wget("-nv",file)
  end
end

# Gets a venue directory from DBLP
$root="http://www.informatik.uni-trier.de/~ley/db/"
$dir="conf/"

def wget_dblp_venue(args,name) 
  # -nH removes the server name
  # -np specifies to get only those files under the given URL
  # --cut-dirs= ignores the first NUM subdirs
  return wget("-nH -np --cut-dirs=3 "+args,$root + $dir + name)
end

# returns the proper DBLP abbreviation for the venue, first assuming it's
# a conference, then a journal
def find_venue(venue)
  urls = [venue,venue.downcase]
  urls.each do |name|
     if wget_dblp_venue("-q -O /dev/null -o /dev/null", name + "/index.html") 
       return name
     end
  end
  if $dir == "journals/" then
    print "DBLP does not seem to track the #{venue} venue (conference or journal).\n Check alternative abbreviations\n"
    exit 1
  else
    $dir = "journals/"
    return find_venue(venue)
  end
end

def get_paper_data(venue,indexes)
  urls = []
  indexes.each do |file|
    File.open(file.chomp!,"r") do |h|
      #puts "processing #{file}"
      while line = h.gets do
        #puts "processing #{line}"
        regexp_str = "href=\"http://dblp.uni-trier.de/rec/bibtex/" + $dir + venue + "/[A-Z]\\w*\\.xml"
        re = Regexp.new(regexp_str)
        if line =~ re then
          s = $&
          urls.push(s[6..s.length])
        end
      end
    end
  end
  return urls
end

################ MAIN PROGRAM ##################

def usage() 
  puts "usage : $0 <abbreviation>"
  exit 1
end

if ARGV.length != 1 then
  usage()
end

# Find the venue at DBLP; first assume it's a conf, then a journal
venue = ARGV[0]
venue = find_venue(venue)

# Make sure directory etc. doesn't already exist
if File.exists?("./"+venue) then
  print "Cannot download information as file ./#{venue} exists. Remove and retry.\n"
  exit 2
else
  print "Downloading indexes ...\n"
end

# Get author information
# argument -r specifies to get the entire directory structure
if !wget_dblp_venue("-r -nv",venue+"/") then
  print "Failed to download indexes.  May need to clean up; remove the #{venue} directory\n".
  exit(3)
end

Dir.chdir(venue)

# Now get the metadata for each paper

indexes = `\ls #{venue}*.html`
urls = get_paper_data(venue,indexes)
wget_files(urls)
Dir.chdir("..")

puts "***************************\n"
puts "From this directory, run getauthors.rb #{venue} to output author data"
