#!/usr/bin/ruby
#
# usage: process.rb <db> [db2 db3 ...]
# where <db> (and likewise db2, db3, etc.) contains the output from the
# getauthors.rb script, or this script (which outputs in the same format)

# Set this to true if you want to filter the authors of the processed
# files according to some criteria of your choice
filter = false

# This should always be false (used for debugging)
compressed = false

# Reads in each file given on the command line, and stores data in a
# hash, to associate each author with one or more papers
results = { }
ARGV.each do |file|
  File.open(file,"r") do |h|
    #puts "processing #{file}"
    while line = h.gets do
      #puts "processing #{line}"
      toks = line.chomp.split(/,\s/)
      key = toks[0]
      if key != nil then
        if results[key] == nil then
          results[key] = { }
        end
        toks[1..toks.length].each do |tok|
          tag,val = tok.split(/:/)
          cur = results[key][tag]
          if (cur == nil)
            results[key][tag] = [val]
          else
            if (tag != "LOC") || (val != cur[0]) then
              cur.concat([val])
            end
          end
        end
      end
    end
  end
end

# filtering; removes authors according to various crtieria

if (filter) then
  results.keys.each do |name|
    vals = results[name]
    poplorpldi = false
    recent_pc = false
    verif = false
    papers = 0
    latest_paper = 0
    earliest_paper = 2010
    vals.keys.each do |tag|
      # count the total papers
      if (tag == 'POPL' || \
          tag == 'VMCAI' || \
          tag == 'OOPSLA' || \
          tag == 'ESOP' || \
          tag == 'CAV' || \
          tag == 'LICS' || \
          tag == 'PLDI' || \
          tag == 'ESOP' || \
          tag == 'ECOOP' || \
          tag == 'FSE' || \
          tag == 'ISSTA' || \
          tag == 'ICSE' || \
          tag == 'ICFP') then
        years = vals[tag]
        papers = papers + years.length
        years.each do |year|
          y = year.to_i + 2000
          if y > latest_paper then
            latest_paper = y
          end
          if y < earliest_paper then
            earliest_paper = y
          end
        end
      end
      # verification focus
      if (tag == 'VMCAI' ||
          tag == 'CAV') then
        verif = true
      end
      # whether they have at least one popl paper
      if (tag == 'POPL' || tag == 'PLDI')
        poplorpldi = true
      # whether they were recently on the popl pc
      elsif (tag == 'PC')
        vals[tag].each do |pc|
          pc =~ /POPL-(\d\d)/
          year = $1
          if year != nil then
            year = year.to_i + 2000
            #puts "#{name} was on POPL #{year}"
            if (year > 2008) 
              recent_pc = true
            end
          end
        end
      end
    end
    if !poplorpldi || latest_paper < 2010 || earliest_paper > 2006 || papers < 3 then
      #puts "Removing #{name} (popl=#{poplorpldi}, recent_pc=#{recent_pc})"
      results.delete(name)
    end
  end
end

# output

# special output for debugging
if (compressed) then
  results.keys.sort.each do |name|
    puts name
    vals = results[name]
    vals.keys.sort.each do |tag|
      puts "  #{tag}: #{vals[tag].sort.to_s}"
    end
  end
else

# normal output format
  results.keys.sort.each do |name|
    acc = name
    #puts name
    vals = results[name]
    vals.keys.sort.each do |tag|
      #puts "--#{tag}"
      vals[tag].each { |val|
        #puts "--#{val}"
        acc = acc + ", " + tag + ":" + val
      }
    end
    puts acc
  end
end
  
